use crate::cmd::config::app_config;
use crate::ems::{Consumer, Producer, Service};
use crate::internal::utils;
use crate::internal::utils::mqtt::{mqtt_client, MqttClient};
use anyhow::anyhow;
use log::{debug, info};
use socketcan::tokio::{AsyncCanSocket, CanSocket};
use socketcan::{EmbeddedFrame, IoResult};
use std::io::Read;
use std::sync::Arc;
use tokio::sync::Mutex;

pub struct Bms {
    pub id: String,
    pub producer: Producer,
    pub consumer: Arc<Mutex<Consumer>>,
    pub can_socket: Arc<AsyncCanSocket<socketcan::CanSocket>>,
}

impl Bms {
    /// 初始化BMS
    pub async fn new(producer: Producer, consumer: Consumer) -> anyhow::Result<Self> {
        let id = utils::generate_random_str(12);
        utils::log::init_log("inpower_iot_mgc_rs::ems::bms::*", "bms/bms.log").await;
        info!("BMS [{}] 初始化成功", id);
        let can_socket = match CanSocket::open(app_config().emu.bms.name.as_str()) {
            Ok(can) => can,
            Err(e) => {
                return Err(anyhow!(
                    "can: {}打开失败: {}",
                    app_config().emu.bms.name.as_str(),
                    e.to_string()
                ));
            }
        };
        Ok(Bms {
            id,
            producer,
            consumer: Arc::new(Mutex::new(consumer)),
            can_socket: Arc::new(can_socket),
        })
    }
}

#[async_trait::async_trait]
impl Service for Bms {
    async fn read_task(&self) {
        let producer = self.producer.clone();
        let can_socket = Arc::clone(&self.can_socket);
        loop {
            let can_frame = can_socket.read_frame().await.unwrap();
            debug!("{:?}", can_frame)
        }
    }

    async fn write_task(&self) {
        let consumer = self.consumer.clone();
        loop {
            let string = consumer.lock().await.recv().await.unwrap();
            info!("[BMS] {}", string);
        }
    }
}
