use crate::cmd::config::app_config;
use amqprs::connection::{Connection, OpenConnectionArguments};
use std::sync::OnceLock;

/// RabbitMQ 单例
static MQ: OnceLock<RabbitMQ> = OnceLock::new();

pub async fn mq_conn() -> anyhow::Result<&'static RabbitMQ> {
    let conn = match MQ.get() {
        Some(mq_conn) => mq_conn,
        None => {
            let result = RabbitMQ::new().await?;
            MQ.set(result)
                .map_err(|_| anyhow::anyhow!("MQ already initialized"))?;
            MQ.get().unwrap()
        }
    };
    Ok(conn)
}

pub struct RabbitMQ {
    pub conn: Connection,
}

impl RabbitMQ {
    pub async fn new() -> Result<Self, anyhow::Error> {
        let config = &app_config().rabbitmq;
        let conn = Connection::open(&OpenConnectionArguments::new(
            config.host.as_str(),
            config.port,
            config.username.as_str(),
            config.password.as_str(),
        ))
        .await?;
        Ok(RabbitMQ { conn })
    }
}
